static sym
const latin3_syms[] = {
	{ 0x00a0, "nobreakspace" },
	{ 0x0126, "Hstroke" },
	{ 0x02d8, "breve" },
	{ 0x00a3, "sterling" },
	{ 0x00a4, "currency" },
	{ 0xfffd, "" },
	{ 0x0124, "Hcircumflex" },
	{ 0x00a7, "section" },
	{ 0x00a8, "diaeresis" },
	{ 0x0130, "Iabovedot" },
	{ 0x015e, "Scedilla" },
	{ 0x011e, "Gbreve" },
	{ 0x0134, "Jcircumflex" },
	{ 0x00ad, "hyphen" },
	{ 0xfffd, "" },
	{ 0x017b, "Zabovedot" },
	{ 0x00b0, "degree" },
	{ 0x0127, "hstroke" },
	{ 0x00b2, "twosuperior" },
	{ 0x00b3, "threesuperior" },
	{ 0x00b4, "acute" },
	{ 0x00b5, "mu" },
	{ 0x0125, "hcircumflex" },
	{ 0x00b7, "periodcentered" },
	{ 0x00b8, "cedilla" },
	{ 0x0131, "idotless" },
	{ 0x015f, "scedilla" },
	{ 0x011f, "gbreve" },
	{ 0x0135, "jcircumflex" },
	{ 0x00bd, "onehalf" },
	{ 0xfffd, "" },
	{ 0x017c, "zabovedot" },
	{ 0x00c0, "Agrave" },
	{ 0x00c1, "Aacute" },
	{ 0x00c2, "Acircumflex" },
	{ 0xfffd, "" },
	{ 0x00c4, "Adiaeresis" },
	{ 0x010a, "Cabovedot" },
	{ 0x0108, "Ccircumflex" },
	{ 0x00c7, "Ccedilla" },
	{ 0x00c8, "Egrave" },
	{ 0x00c9, "Eacute" },
	{ 0x00ca, "Ecircumflex" },
	{ 0x00cb, "Ediaeresis" },
	{ 0x00cc, "Igrave" },
	{ 0x00cd, "Iacute" },
	{ 0x00ce, "Icircumflex" },
	{ 0x00cf, "Idiaeresis" },
	{ 0xfffd, "" },
	{ 0x00d1, "Ntilde" },
	{ 0x00d2, "Ograve" },
	{ 0x00d3, "Oacute" },
	{ 0x00d4, "Ocircumflex" },
	{ 0x0120, "Gabovedot" },
	{ 0x00d6, "Odiaeresis" },
	{ 0x00d7, "multiply" },
	{ 0x011c, "Gcircumflex" },
	{ 0x00d9, "Ugrave" },
	{ 0x00da, "Uacute" },
	{ 0x00db, "Ucircumflex" },
	{ 0x00dc, "Udiaeresis" },
	{ 0x016c, "Ubreve" },
	{ 0x015c, "Scircumflex" },
	{ 0x00df, "ssharp" },
	{ 0x00e0, "agrave" },
	{ 0x00e1, "aacute" },
	{ 0x00e2, "acircumflex" },
	{ 0xfffd, "" },
	{ 0x00e4, "adiaeresis" },
	{ 0x010b, "cabovedot" },
	{ 0x0109, "ccircumflex" },
	{ 0x00e7, "ccedilla" },
	{ 0x00e8, "egrave" },
	{ 0x00e9, "eacute" },
	{ 0x00ea, "ecircumflex" },
	{ 0x00eb, "ediaeresis" },
	{ 0x00ec, "igrave" },
	{ 0x00ed, "iacute" },
	{ 0x00ee, "icircumflex" },
	{ 0x00ef, "idiaeresis" },
	{ 0xfffd, "" },
	{ 0x00f1, "ntilde" },
	{ 0x00f2, "ograve" },
	{ 0x00f3, "oacute" },
	{ 0x00f4, "ocircumflex" },
	{ 0x0121, "gabovedot" },
	{ 0x00f6, "odiaeresis" },
	{ 0x00f7, "division" },
	{ 0x011d, "gcircumflex" },
	{ 0x00f9, "ugrave" },
	{ 0x00fa, "uacute" },
	{ 0x00fb, "ucircumflex" },
	{ 0x00fc, "udiaeresis" },
	{ 0x016d, "ubreve" },
	{ 0x015d, "scircumflex" },
	{ 0x02d9, "abovedot" },
};
