#include "config.h"
#include "modifiers.h"

const modifier_t modifiers[] = {
	{ "shift", KG_SHIFT },
	{ "altgr", KG_ALTGR },
	{ "control", KG_CTRL },
	{ "alt", KG_ALT },
	{ "shiftl", KG_SHIFTL },
	{ "shiftr", KG_SHIFTR },
	{ "ctrll", KG_CTRLL },
	{ "ctrlr", KG_CTRLR },
	{ "capsshift", KG_CAPSSHIFT },
	{ 0, 0 }
};
